/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { SGraphyItem } from "@saga-web/graphy/";
import { SMouseEvent } from "@saga-web/base/";;
import {
    SColor,
    SLineCapStyle,
    SPainter,
    SPath2D,
    SPoint,
    SPolygonUtil,
    SRect,
    SLine
} from "@saga-web/draw";
import { PolygonData } from "./type/PolygonData";
import { Point } from "./type/Point";
/**
 * 编辑多边形
 *
 * @author  韩耀龙
 */
export class SPolygonItem extends SGraphyItem {
    /** X坐标最小值  */
    minX = Number.MAX_SAFE_INTEGER;
    /** X坐标最大值  */
    maxX = Number.MIN_SAFE_INTEGER;
    /** Y坐标最小值  */
    minY = Number.MAX_SAFE_INTEGER;
    /** Y坐标最大值  */
    maxY = Number.MIN_SAFE_INTEGER;
    /** 轮廓线坐标  */
    pointList: SPoint[] = [];
    /** 是否闭合    */
    closeFlag: boolean = false;
    // 当前状态 1：show 2 创建中，3 编辑中
    _status: number = 2;
    zOrder: number = 1000;
    /** 鼠标移动点  */
    private lastPoint = new SPoint();
    constructor(parent: SGraphyItem | null, data: PolygonData | null) {
        super(parent);
        if (data) {
            if (data.PointList) {
                this._xyzListToSPointList(data.PointList);
            }
        } else {
            // 如果
            this.pointList = [];
            this.update();
        }
    }
    get getStatus(): number {
        return this._status;
    }
    set setStatus(value: number) {
        this._status = value;
        this.update();
    }

    /**
 * 新增多边形顶点
 * @param x   点位得x坐标
 * @param y   点位得y坐标
 * @param i   点位得i坐标
 * @return SPoint
 */
    addPoint(x: number, y: number, i?: number): SPoint {
        const point = new SPoint(x, y);
        if (typeof i == 'undefined') {
            this.pointList.push(point)
        } else {
            this.pointList.splice(i, 0, point);
        }
        this.update()
        return point
    }

    /** 删除点位
* 新增多边形顶点
* @param i   点位得i坐标
* @return SPoint
*/
    deletePoint(i?: number): SPoint | null | undefined {
        let point = null;
        if (typeof i != 'undefined') {
            if (this.pointList.length - 1 >= i) {
                point = this.pointList[i];
                this.pointList.splice(i, 0);
            } else {
                point = null
            }
        } else {
            point = this.pointList.pop();
        }
        return point
    }
    /** 移动点位
* 新增多边形顶点
* @param x   点位得x坐标
* @param y   点位得y坐标
* @param i   点位得i坐标
* @return SPoint
*/
    movePoint(x: number, y: number, i: number, ): SPoint | null | undefined {
        let point = null;
        if (this.pointList.length) {
            this.pointList[i].x = x;
            this.pointList[i].y = y;
        }
        point = this.pointList[i];
        return point
    }
    PrintList() {
    }

    /**
     * Item对象边界区域
     *
     * @return SRect
     */
    boundingRect(): SRect {
        return new SRect(
            this.minX,
            this.minY,
            this.maxX - this.minX,
            this.maxY - this.minY
        );
    } // Function boundingRect()

    /**
    * 构造函数
    *
    * @param arr    外层传入的数据PointList
    */
    _xyzListToSPointList(arr: Point[]) {
        if (arr.length) {
            this.pointList = arr.map(it => {
                let x = it.X,
                    y = it.Y;
                if (x < this.minX) {
                    this.minX = x;
                }
                if (y < this.minY) {
                    this.minY = y;
                }
                if (x > this.maxX) {
                    this.maxX = x;
                }
                if (y > this.maxY) {
                    this.maxY = y;
                }
                return new SPoint(x, y);
            });
        } else {
            this.pointList = []
        }
    }
    /**
   * 构造函数  闭合多边形
   *
   */
    _closePolygon(painter: SPainter, pointList: SPoint[], closeFlag: boolean = false) {
        this._status = 1;
        painter.pen.lineCapStyle = SLineCapStyle.Square;
        painter.pen.color = new SColor("#0091FF");
        painter.pen.lineWidth = painter.toPx(4);
        painter.drawPolyline(this.pointList);
        // 是否闭合
        if (closeFlag) {
            painter.drawLine(
                pointList[this.pointList.length - 1],
                pointList[0]
            );
        }
    }

    /**
 * 构造函数  创建多边形
 *
 */
    createPolygon(painter: SPainter, pointList: SPoint[], closeFlag: boolean = false) {
        painter.pen.lineCapStyle = SLineCapStyle.Square;
        painter.pen.color = new SColor("#0091FF");
        painter.pen.lineWidth = painter.toPx(4);
        painter.drawPolyline(this.pointList);
        if (this.lastPoint) {
            painter.drawLine(
                pointList[this.pointList.length - 1],
                this.lastPoint
            );
        };
        painter.pen.color = SColor.Transparent;
        painter.pen.lineWidth = painter.toPx(4);
        painter.drawPolygon([...this.pointList, this.lastPoint]);
    }
    ////////////以下为鼠标事件

    /**
    * 鼠标双击事件
    *
    * @param	event         事件参数
    * @return	boolean
    */

    onDoubleClick(event: SMouseEvent): boolean {
        return true;
    } // Function onDoubleClick()

    /**
 * 键盘事件
 *
 * @param	event         事件参数
 * @return	boolean
 */
    onKeyDown(event: KeyboardEvent): boolean {
        console.log(event)
        return true;
    } // Function onKeyDown()
    /**
     * 鼠标双击事件
     *
     * @param	event         事件参数
     * @return	boolean
     */
    onMouseDown(event: SMouseEvent): boolean {
        if (this._status == 2) {
            this.addPoint(event.x, event.y)
        }

        return true;
    } // Function onMouseDown()
    /**
     * 鼠标双击事件
     *
     * @param	event         事件参数
     * @return	boolean
     */
    onMouseEnter(event: SMouseEvent): boolean {
        return true;
    } // Function onMouseDown()
    /**
     * 鼠标双击事件
     *
     * @param	event         事件参数
     * @return	boolean
     */
    onMouseLeave(event: SMouseEvent): boolean {
        return true;
    } // Function onMouseDown()
    onMouseMove(event: SMouseEvent): boolean {
        if (this._status == 2) {
            this.lastPoint.x = event.x;
            this.lastPoint.y = event.y;
        }
        this.update()
        return true;
    } // Function onMouseDown()
    onMouseUp(event: SMouseEvent): boolean {
        return true;
    } // Function onMouseUp()
    onResize(event: SMouseEvent): boolean {
        return true;
    } // Function onMouseUp()


    ////////////////////////

    /**
* Item绘制操作
*
* @param   painter       painter对象
*/
    onDraw(painter: SPainter): void {
        painter.pen.lineCapStyle = SLineCapStyle.Square;
        painter.pen.color = SColor.Transparent;
        painter.pen.lineWidth = painter.toPx(4);
        painter.brush.color = new SColor("#1EE887")
        // 当状态为展示状态
        if (this._status == 1) {
            painter.drawPolygon(this.pointList);
            // 闭合多边形
        } else if (this._status == 2) {
            // 创建状态
            // this._closePolygon(painter, this.pointList, this.closeFlag);
            this.createPolygon(painter, this.pointList)
        } else {
            // 编辑状态
            //   this._closePolygon(painter, this.pointList);
        }
    } // Function onDraw()
}
